#!/usr/bin/perl -w
use strict;

my $posOnly = 0;
while (1) {
    my $tmp = shift or last;
    if ($tmp eq '-pos') { $posOnly = 1; }
    else { die; }
}


my %pcfg = ();
my $lineNum = 0;
my $breakOut = 0;
while (<>) {
    chomp;
    $lineNum++;
    my $line = $_;
    if (/^no parse!/) { next; }
    if (/^([0-9\.-]+)\s+(.+)$/) {
        my $lp = $1;
        my $tree = $2;
        $tree =~ s/\s+/ /g; $tree =~ s/^ //; $tree =~ s/ $//;
        $breakOut = 0;
        my ($tr, $rest) = parseTree($tree);
        if (not $breakOut) {
            if ($rest ne '') { 
                print STDERR "[$lineNum]: Unexpected text at end: \"$rest\"\n";
            } else {
                collectCounts($tr);
            }
        }
    } else {
        print "[$lineNum] Malformed line, cannot split into prob/tree: \"$line\"\n";
    }
}

# normalize CFG and print
foreach my $lhs (sort keys %pcfg) {
    foreach my $rhs (sort { $pcfg{$lhs}{$b} <=> $pcfg{$lhs}{$a} } keys %{$pcfg{$lhs}}) {
        print '' . $pcfg{$lhs}{$rhs} . "\t" . $lhs . "\t" . $rhs . "\n";
    }
}


sub collectCounts {
    my ($tr) = @_;
    if (not exists $tr->{'children'}) { return; }
    if ($posOnly && (scalar @{$tr->{'children'}} == 1) && (not exists $tr->{'children'}[0]{'children'})) { return; }

    my $lhs = $tr->{'lhs'};
    my $rhs = '';
    foreach my $c (@{$tr->{'children'}}) { 
        $rhs .= ' ' . $c->{'lhs'};
    }
    $rhs =~ s/^ //;
    $pcfg{$lhs}{$rhs}++;
    foreach my $c (@{$tr->{'children'}}) { 
        collectCounts($c);
    }
}

sub parseTree {
    ($_) = @_;
    s/^ *//;
    if (s/^\(([^ ]+)//) {
        my $lhs = $1;
        s/^ //;
        my @c = ();
        while (1) {
            my ($child, $rest) = parseTree($_);
            if ($breakOut) { return; }
            $_ = $rest;
            push @c, \%{$child};
            s/ *//;
            if ($_ eq '') { 
                print STDERR "[$lineNum]: Ran out of input while parsing tree\n";
                $breakOut = 1;
                last;
            } elsif (s/^\)//) { 
                last; 
            }
        }
        my %tree = ('lhs' => $lhs, 'children' => \@c);
        return (\%tree, $_);
    } elsif (/^\)/) { 
        print STDERR "[$lineNum]: Got unexpected close-paren in \"$_\"\n";
        $breakOut = 1;
        return;
    } elsif (s/^([^ \)]+) *//) {
        my %tree = ('lhs' => $1);
        return (\%tree, $_);
    }
}
