#!/usr/bin/perl -w
use strict;

my $lineNum = 0;
while (<>) {
    chomp;
    $lineNum++;
    my $line = $_;
    if (/^no parse!/) { print "no parse!\n"; next; }
    if (/^([0-9\.-]+)\s+(.+)$/) {
        my $lp = $1;
        my $tree = $2;
        $tree =~ s/\s+/ /g; $tree =~ s/^ //; $tree =~ s/ $//;
        my ($tr, $rest) = parseTree($tree);
        if ($rest ne '') { die; }
        debinarizeTree($tr);
        print $lp . "\t"; printTree($tr); print "\n";
    } else { die; }
}

sub debinarizeTree {
    my ($tr) = @_;
#    print STDERR "debinarizeTree(" . $tr->{'lhs'} . ")\n";
    if ($tr->{'lhs'} =~ /__/) { 
        print STDERR "[$lineNum] Warning: root of tree has been binarized; defaulting to 'S'\n";
        $tr->{'lhs'} = 'S';
    }
    if (not exists $tr->{'children'}) { return $tr; }
    my @c = @{$tr->{'children'}};
    if (@c == 1) {
        my $lhs = $c[0]->{'lhs'};
        if ($lhs =~ /__/) {
            my $lhsNew = $lhs; $lhsNew =~ s/__.*$//;
            print STDERR "[$lineNum] Warning: unary production leads to binarized node '$lhs', defaulting to '$lhsNew'\n";
            $c[0]->{'lhs'} = $lhsNew;
        }
        $c[0] = debinarizeTree($c[0]);
    } else {
        while (1) {
            my $c = pop @c;  # get last child
            if ($c->{'lhs'} =~ /__/) { # need to debinarize
                if (not defined $c->{'children'}) { 
                    print STDERR "[$lineNum] Warning: got a binarized terminal!  egads!\n";
                } else {
                    foreach my $cnew (@{$c->{'children'}}) {
                        push @c, $cnew;
                    }
                }
            } else { 
                push @c, $c;
                last;
            }
        }
        for (my $i=0; $i<@c; $i++) {
            $c[$i] = debinarizeTree($c[$i]);
        }
    }

    $tr->{'children'} = \@c;
    return $tr;
}


sub parseTree {
    ($_) = @_;
    s/^ *//;
    if (s/^\(([^ ]+)//) {
        my $lhs = $1;
        s/^ //;
        my @c = ();
        while (1) {
            my ($child, $rest) = parseTree($_);
            $_ = $rest;
            push @c, \%{$child};
            s/ *//;
            if ($_ eq '') {  die; }
            elsif (s/^\)//) { 
                last; 
            }
        }
        my %tree = ('lhs' => $lhs, 'children' => \@c);
        return (\%tree, $_);
    } elsif (/^\)/) { 
        die;
    } elsif (s/^([^ \)]+) *//) {
        my %tree = ('lhs' => $1);
        return (\%tree, $_);
    }
}

sub printTree {
    my ($tr) = @_;
    if (not exists $tr->{'children'}) {
        print $tr->{'lhs'};
    } else {
        my @c = @{$tr->{'children'}};
        print '(' . $tr->{'lhs'};
        foreach my $c (@c) {
            print ' ';
            printTree($c);
        }
        print ')';
    }
}
