#!/usr/bin/perl -w
use strict;

my $lineNum = 0;
my $breakOut = 0;
while (<>) {
    chomp;
    $lineNum++;
    my $line = $_;
    if (/^no parse!/) { next; }
    if (/^([0-9\.-]+)\s+(.+)$/) {
        my $lp = $1;
        my $tree = $2;
        $tree =~ s/\s+/ /g; $tree =~ s/^ //; $tree =~ s/ $//;
        $breakOut = 0;
        my ($tr, $rest) = parseTree($tree);
        if (not $breakOut) {
            if ($rest ne '') { 
                print STDERR "[$lineNum]: Unexpected text at end: \"$rest\"\n";
            } else {
                print $lp . "\t";
                printTree($tr);
                print "\n";
            }
        }
    } else {
        print "[$lineNum] Malformed line, cannot split into prob/tree: \"$line\"\n";
    }
}

sub parseTree {
    ($_) = @_;
    s/^ *//;
    if (s/^\(([^ ]+)//) {
        my $lhs = $1;
        s/^ //;
        my @c = ();
        while (1) {
            my ($child, $rest) = parseTree($_);
            if ($breakOut) { return; }
            $_ = $rest;
            push @c, \%{$child};
            s/ *//;
            if ($_ eq '') { 
                print STDERR "[$lineNum]: Ran out of input while parsing tree\n";
                $breakOut = 1;
                last;
            } elsif (s/^\)//) { 
                last; 
            }
        }
        my %tree = ('lhs' => $lhs, 'children' => \@c);
        return (\%tree, $_);
    } elsif (/^\)/) { 
        print STDERR "[$lineNum]: Got unexpected close-paren in \"$_\"\n";
        $breakOut = 1;
        return;
    } elsif (s/^([^ \)]+) *//) {
        my %tree = ('lhs' => $1);
        return (\%tree, $_);
    }
}

sub printTree {
    my ($tr) = @_;
    if (not exists $tr->{'children'}) {
        print $tr->{'lhs'};
    } else {
        my @c = @{$tr->{'children'}};
        print '(' . $tr->{'lhs'};
        foreach my $c (@c) {
            print ' ';
            printTree($c);
        }
        print ')';
    }
}
